# -*- coding: utf-8 -*-

import platform
import sys
import getopt
import pykakasi

try:
    from importlib import metadata as importlib_metadata  # noqa
except ImportError:
    import importlib_metadata


def show_version():
    dist = importlib_metadata.distribution('pykakasi')
    py_version = platform.python_version()
    py_impl = platform.python_implementation()
    py_build = platform.python_compiler()
    print("pykakasi: version {} on Python {} [{} {}]".format(dist.version, py_version, py_impl, py_build))


def usage():
    print("Python implementation of kakasi\n")
    print("{}".format(pykakasi.__copyright__))
    print("URL: {}".format(pykakasi.__url__))


def do_wakati(mode, infile, outfile):
    kakasi = pykakasi.wakati()
    do_it(kakasi, mode, infile, outfile)

def do_kakasi(mode, infile, outfile):
    kakasi = pykakasi.kakasi()
    do_it(kakasi, mode, infile, outfile)

def do_it(kakasi, mode, infile, outfile):
    try:
        for k,v in mode.iteritems():
            kakasi.setMode(k,v)
    except:
        for k,v in mode.items():
            kakasi.setMode(k,v)
    converter  = kakasi.getConverter()
    if infile:
        infp = open(infile,'r')
    else:
        infp = sys.stdin
    if outfile:
        outfp = open(outfile,'w')
    else:
        outfp = sys.stdout
    for line in infp:
        outfp.write(converter.do(line))

def main():
    outfile = None
    infile  = None
    wakati = False
    mode = {}
    values = ("J","H","K","E","a")
    romanvals = {"h":"Hepburn", "k":"Kunrei", "p":"Passport"}

    try:
        opts, args = getopt.getopt(sys.argv[1:],
            "hvI:O:wsS:J:H:K:E:a:r:fC",
            ["help","version","input=","output=","wakati","space","separator="])
    except getopt.GetoptError as err:
        print (err)
        usage()
        sys.exit(2)

    for o, a in opts:
        if o in ["-v","--version"]:
            show_version()
            sys.exit()
        elif o in ("-h", "--help"):
            show_version()
            usage()
            sys.exit()
        elif o in ("-I", "--input"):
            infile = a
        elif o in ("-O", "--output"):
            outfile = a
        elif o in ("-w", "--wakati"):
            wakati = True
        elif o in ("-s","--space"):
            mode["s"] = True
        elif o in ["-S","--separator"]:
            mode["S"] = a
        elif o == "-r":
            if a in romanvals.keys():
                mode['r'] = romanvals[a]
        elif o in ("-J","-H","-K","-E","-a"):
            if a in values:
                mode[o[1:]] = a
        elif o in ("-f","-C","-U"):
            mode[o[1:]] = True
        else:
            assert False, "unhandled option"

    if wakati:
        do_wakati(mode, infile, outfile)
    else:
        do_kakasi(mode, infile, outfile)

if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011-2018 Hiroshi Miura <miurahr@linux.com>
#

__license__ = 'GPL 3'
__copyright__ = '2014-2020 Hiroshi Miura <miurahr@linux.com>'
__docformat__ = 'restructuredtext en'

from typing import Dict, List, Optional, Union

from .exceptions import (InvalidFlagValueException, InvalidModeValueException,
                         UnknownOptionsException,
                         UnsupportedRomanRulesException)
from .kanji import J2
from .properties import Ch
from .scripts import A2, H2, K2, Sym2


class kakasi:

    _keys = ["J", "H", "K", "E", "a"]  # type: List[str]
    _values = ["a", "E", "H", "K"]  # type: List[str]
    _roman_vals = ["Hepburn", "Kunrei", "Passport"]  # type: List[str]
    _MAXLEN = 32  # type: int
    _LONG_SYMBOL = [
        # 0x002D,  # -
        0x30FC,  # ー
        # 0x2010,  # ‐
        # 0x2011,  # ‑
        # 0x2013,  # –
        # 0x2014,  # —
        0x2015,  # ―
        0x2212,  # −
        0xFF70  # ｰ
    ]  # type: List[int]

    def __init__(self):
        self._conv = {}  # type: Dict[str, Union[J2, H2, K2, A2, Sym2]]
        self._mode = {"J": None, "H": None, "K": None, "E": None, "a": None}  # type: Dict[str, Optional[str]]
        self._furi = {"J": False, "H": False, "K": False, "E": False, "a": False}  # type: Dict[str, bool]
        self._flag = {"p": False, "s": False, "f": False, "c": False, "C": False, "U": False,
                      "u": False, "t": True}  # type: Dict[str, bool]
        self._option = {"r": "Hepburn"}  # type: Dict[str, str]
        self._separator = ' '  # type: str
        self._separator_string = ' '  # type: str
        self._jconv = J2("H")
        self._hahconv = H2("a", method='Hepburn')
        self._hakconv = H2("a", method='Kunrei')
        self._hapconv = H2("a", method='Passport')
        self._hkconv = H2("K")
        self._khconv = K2("H")
        self._kaconv = K2("a")
        self._aeconv = A2("E")
        self._saconv = Sym2("a")

    def convert(self, text: str) -> List[Dict[str, str]]:
        _state = True

        if len(text) == 0:
            return [{'orig': "", 'kana': "", 'hira': "", 'hepburn': "", 'passport': "", 'kunrei': ""}]

        otext = ''
        _result = []
        i = 0

        while i < len(text):
            if self._jconv.isRegion(text[i]):
                t, ln = self._jconv.convert(text[i:])
                if ln <= 0:  # pragma: no cover
                    otext = otext + text[i]
                    i += 1
                    _state = False
                else:
                    if _state:
                        _result.append(self._iconv(otext + text[i:i + ln], t))
                    else:
                        _result.append(self._iconv(otext, otext))
                        _result.append(self._iconv(text[i:i + ln], t))
                        _state = True
                    otext = ''
                    i = i + ln
            else:
                _state = False
                otext = otext + text[i]
                i += 1

                if ord(otext[-1]) in Ch.endmark:
                    _result.append(self._iconv(otext, otext))
                    otext = ''
                    _state = True

        if otext:
            _result.append(self._iconv(otext, otext))

        return _result

    def _iconv(self, otext: str, hira: str) -> Dict[str, str]:
        kana = self._h2k(hira)
        hira = self._k2h(hira)  # make sure hiragana doesn't contain katakana
        tmp = {'orig': otext, 'hira': hira, 'kana': kana, 'hepburn': self._s2a(self._h2ah(hira)),
               'kunrei': self._s2a(self._h2ak(hira)), 'passport': self._s2a(self._h2ap(hira))}
        return tmp

    def _s2a(self, text: str) -> str:
        result = ''  # type: str
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))  # type: int
            (t, l1) = self._saconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            elif ord(text[i]) in self._LONG_SYMBOL:  # handle chōonpu sound marks
                # use previous char as a transliteration for kana-dash
                if len(result) > 0:
                    result += result[-1]
                else:
                    result += '-'
                i += 1
            else:
                result += text[i:i + 1]
                i += 1
        return result

    def _k2h(self, text: str) -> str:
        result = ''
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._khconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i:i + 1]
                i += 1
        return result

    def _h2k(self, text: str) -> str:
        result = ''
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hkconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i:i + 1]
                i += 1
        return result

    def _h2ak(self, text: str) -> str:
        result = ''
        i = 0
        while i < len(text):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hakconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i:i + 1]
                i += 1
        return result

    def _h2ah(self, text: str) -> str:
        result = ''
        i = 0
        while(i < len(text)):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hahconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i:i + 1]
                i += 1
        return result

    def _h2ap(self, text: str) -> str:
        result = ''
        i = 0
        while(i < len(text)):
            w = min(i + self._MAXLEN, len(text))
            (t, l1) = self._hapconv.convert(text[i:w])
            if l1 > 0:
                result += t
                i += l1
            else:
                result += text[i:i + 1]
                i += 1
        return result

    def setMode(self, fr: str, to: Optional[Union[bool, str]]) -> None:
        if fr in self._keys:
            if to is None:
                self._mode[fr] = None
            elif isinstance(to, str) and to[0] in self._values:
                self._mode[fr] = to[0]
                if len(to) == 2 and to[1] == "F":
                    self._furi[fr] = True
            else:
                raise InvalidModeValueException("Invalid value for mode")
        elif fr in self._flag.keys():
            if isinstance(to, bool):
                self._flag[fr] = to
            else:
                raise InvalidFlagValueException("Invalid flag value")
        elif fr == "r":
            if isinstance(to, str) and to in self._roman_vals:
                self._option["r"] = to
            else:
                raise UnsupportedRomanRulesException("Unknown roman table name")
        elif fr == "S":
            if isinstance(to, str):
                self._separator = to
            else:
                raise InvalidFlagValueException("Incompatible separator value")
        else:
            raise UnknownOptionsException("Unhandled options")  # pragma: no cover

    def getConverter(self):
        self._conv["J"] = J2(self._mode["J"], method=self._option["r"])
        self._conv["H"] = H2(self._mode["H"], method=self._option["r"])
        self._conv["K"] = K2(self._mode["K"], method=self._option["r"])
        self._conv["E"] = Sym2(self._mode["E"])
        self._conv["a"] = A2(self._mode["a"])
        return self

    def do(self, text: str) -> str:
        otext = ""
        i = 0
        while True:
            if i >= len(text):
                break

            if self._conv["J"].isRegion(text[i]):
                mode = "J"
            elif self._conv["H"].isRegion(text[i]):
                mode = "H"
            elif self._conv["K"].isRegion(text[i]):
                mode = "K"
            elif self._conv["E"].isRegion(text[i]):
                mode = "E"
            elif self._conv["a"].isRegion(text[i]):
                mode = "a"
            else:
                mode = "o"

            if mode in ("J", "E"):
                w = min(i + self._MAXLEN, len(text))
                (t, l1) = self._conv[mode].convert(text[i:w])

                if l1 > 0:
                    orig = text[i:i + l1]
                    chunk = t
                    i += l1
                else:
                    orig = text[i:i + 1]
                    if self._flag["t"]:
                        chunk = orig
                    else:
                        chunk = "???"
                    i += 1

            elif mode in ("H", "K", "a"):
                orig = ''
                chunk = ''

                while i < len(text):

                    if ord(text[i]) in self._LONG_SYMBOL:

                        # FIXME: q&d workaround when hiragana/katanaka dash is first char.
                        if self._mode[mode] is not None and len(chunk) > 0:
                            # use previous char as a transliteration for kana-dash
                            orig += text[i]
                            chunk = chunk + chunk[-1]
                            i += 1
                        elif len(chunk) == 0:
                            orig += text[i]
                            chunk += '-'
                            i += 1
                            break
                        else:
                            orig += text[i]
                            chunk += text[i]
                            i += 1
                            break

                    elif self._conv[mode].isRegion(text[i]):
                        w = min(i + self._MAXLEN, len(text))
                        (t, l1) = self._conv[mode].convert(text[i:w])
                        if l1 > 0:
                            orig += text[i:i + l1]
                            chunk += t
                            i += l1
                        else:
                            orig = text[i:i + 1]
                            if self._flag["t"]:
                                chunk = orig
                            else:
                                chunk = "???"
                            i += 1
                            break

                    else:
                        # i += 1
                        break

            else:
                otext += text[i]
                i += 1
                continue

            if mode in ("J", "E"):
                if self._flag["U"]:
                    chunk = chunk.upper()
                elif self._flag["C"]:
                    chunk = chunk.capitalize()

            if mode in self._keys and self._furi[mode]:
                otext += orig + "[" + chunk + "]"
            else:
                otext += chunk

            # insert separator when option specified and it is not a last character and not an end mark
            if self._flag["s"] and otext[-len(self._separator):] != self._separator \
                    and i < len(text) and not (ord(text[i]) in Ch.endmark):
                otext += self._separator

        return otext


class wakati(kakasi):

    def __init__(self):
        super(wakati, self).__init__()
        self._state = True  # type: bool

    def getConverter(self):
        return self

    def setMode(self, fr: str, to: Optional[Union[bool, str]]) -> None:
        if fr in self._flag.keys():
            if isinstance(to, bool):
                self._flag[fr] = to
            else:
                raise InvalidFlagValueException("Invalid flag value")
            raise UnknownOptionsException("Unhandled options")

    def do(self, text: str) -> str:

        if len(text) == 0:
            return ""

        otext = ''
        i = 0
        while True:
            if i >= len(text):
                break

            if self._jconv.isRegion(text[i]):
                _, ln = self._jconv.convert(text[i:])
                if ln <= 0:  # pragma: no cover
                    otext = otext + text[i]
                    i += 1
                    self._state = False
                elif (i + ln) < len(text):
                    if self._state:
                        otext = otext + text[i:i + ln] + self._separator
                    else:
                        otext = otext + self._separator + text[i:i + ln] + self._separator
                        self._state = True
                    i = i + ln
                else:
                    if self._state:
                        otext = otext + text[i:i + ln]
                    else:  # pragma: no cover
                        otext = otext + self._separator + text[i:i + ln]
                    break

            else:
                self._state = False
                otext = otext + text[i]
                i += 1

        return otext



# -*- coding: utf-8 -*-
#  exceptions.py
#
# Copyright 2015-2019 Hiroshi Miura <miurahr@linux.com>


class PyKakasiException(Exception):
    pass


class UnknownCharacterException(PyKakasiException):
    pass


class UnsupportedRomanRulesException(PyKakasiException):
    pass


class UnknownOptionsException(PyKakasiException):
    pass


class InvalidModeValueException(PyKakasiException):
    pass


class InvalidFlagValueException(PyKakasiException):
    pass



#  tests for pykakasi-2.0.4-py36ha15d459_1 (this is a generated file);
print('===== testing package: pykakasi-2.0.4-py36ha15d459_1 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
# the upstream session-scoped test fixture redundantly re-generates
# the dictionaries that are already packaged, see
# https://github.com/miurahr/pykakasi/blob/v2.0.4/tests/conftest.py
# unless we specify the following environment variable:
import os
os.environ["TOX_ENV_DIR"] = "dummy"

import pytest
pytest.main(["tests/"])
#  --- run_test.py (end) ---

print('===== pykakasi-2.0.4-py36ha15d459_1 OK =====');
print("import: 'pykakasi'")
import pykakasi




# -*- coding: utf-8 -*-
import concurrent.futures
import threading

import pytest

import pykakasi
import pykakasi.kanji

I_TEST = [
    ("菟", "兎"),
    ("菟集", "兎集"),
    ("熙", "煕"),
    ("壱弍", "一二"),
    ("森鷗外", "森鴎外"),
    ("\U0000845B\U000E0101城市", "\U0000845B城市"),
    ("\U0000845B\U000E0100飾区", "\U0000845B飾区")
]


def test_worker_itaiji():
    assert worker_itaiji()


@pytest.mark.thread
def test_thread_itaiji():
    tasks = []
    for i in range(10):
        t = threading.Thread(target=worker_itaiji)
        tasks.append(t)
        t.start()
    for t in tasks:
        t.join()


@pytest.mark.thread
@pytest.mark.skip
def test_threadpool_itaiji():
    with concurrent.futures.ThreadPoolExecutor(3) as texec:
        tasks = [texec.submit(worker_itaiji) for _ in range(10)]
        for task in concurrent.futures.as_completed(tasks):
            if not task.result():
                raise Exception("Failed.")


def worker_itaiji():
    try:
        j = pykakasi.kanji.J2("H")
        for case, result in I_TEST:
            assert j._itaiji.convert(case) == result
    except AssertionError:
        return False
    return True


@pytest.mark.thread
def test_thread_kanwa():
    tasks = []
    for i in range(10):
        t = threading.Thread(target=worker_kanwa)
        tasks.append(t)
        t.start()
    for t in tasks:
        t.join()


@pytest.mark.thread
@pytest.mark.skip
def test_threadpool_kanwa():
    with concurrent.futures.ThreadPoolExecutor(3) as texec:
        tasks = [texec.submit(worker_kanwa) for _ in range(10)]
        for task in concurrent.futures.as_completed(tasks):
            if not task.result():
                raise Exception("Failed.")


def worker_kanwa():
    try:
        k = pykakasi.kanji.Kanwa()
        d = k.load(u"春")
        assert d[u"春"] is not None
    except AssertionError:
        return False
    return True



#
# Author: Mike McKerns (mmckerns @caltech and @uqfoundation)
# Copyright (c) 2018-2020 The Uncertainty Quantification Foundation.
# License: 3-clause BSD.  The full license text is available at:
#  - https://github.com/uqfoundation/pox/blob/master/LICENSE

import pox.__main__
from pox.__main__ import *
__doc__ = pox.__main__.__doc__


if __name__=='__main__':
    import sys
    try:
        func = sys.argv[1]
    except: func = None
    if func:
        try:
            exec('print(%s)' % func)
        except:
            print("Error: incorrect syntax '%s'\n" % func)
            exec('print(%s.__doc__)' % func.split('(')[0])
    else: help()


# End of file 



# -*- coding: utf-8 -*-

import pykakasi


def test_kakasi_legacy_constitution():

    original_text = u"日本国民は、正当に選挙された国会における代表者を通じて行動し、われらとわれらの子孫のために、" \
                    u"諸国民との協和による成果と、わが国全土にわたつて自由のもたらす恵沢を確保し、" \
                    u"政府の行為によつて再び戦争の惨禍が起ることのないやうにすることを決意し、ここに主権が国民に存することを宣言し、" \
                    u"この憲法を確定する。そもそも国政は、国民の厳粛な信託によるものであつて、その権威は国民に由来し、" \
                    u"その権力は国民の代表者がこれを行使し、その福利は国民がこれを享受する。これは人類普遍の原理であり、" \
                    u"この憲法は、かかる原理に基くものである。われらは、これに反する一切の憲法、法令及び詔勅を排除する。"
    result = "Nihonkokumin ha, Seitou ni Senkyo sareta Kokkai niokeru Daihyousha wo Tsuuji te Koudou shi, " \
             "wareratowarerano Shison notameni, Shokokumin tono Kyouwa niyoru Seika to, waga Kuni Zendo " \
             "niwatatsute Jiyuu nomotarasu Keitaku wo Kakuho shi," \
             " Seifu no Koui niyotsute Futatabi Sensou no Sanka ga Okoru kotononaiyaunisurukotowo Ketsui shi, " \
             "kokoni Shuken ga Kokumin ni Sonsu rukotowo Sengen shi, kono Kenpou wo Kakuteisu ru. " \
             "somosomo Kokusei ha, Kokumin no Genshuku na Shintaku niyorumonodeatsute, sono Ken'i ha Kokumin ni " \
             "Yurai shi, sono Kenryoku ha Kokumin no Daihyousha gakorewo Koushi shi, sono Fukuri ha Kokumin " \
             "gakorewo Kyouju suru. koreha Jinruifuhen no Genri deari, kono Kenpou ha, kakaru Genri ni Motozuku " \
             "monodearu. wareraha, koreni Hansu ru Issai no Kenpou, Hourei Oyobi Shouchoku wo Haijo suru."

    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", "a")
    kakasi.setMode("K", "a")
    kakasi.setMode("J", "a")
    kakasi.setMode("E", "a")
    kakasi.setMode("r", "Hepburn")
    kakasi.setMode("C", True)
    kakasi.setMode("s", True)
    converter = kakasi.getConverter()
    assert converter.do(original_text) == result


def test_wakati_constitution():

    original_text = u"日本国民は、正当に選挙された国会における代表者を通じて行動し、われらとわれらの子孫のために、" \
                    u"諸国民との協和による成果と、わが国全土にわたつて自由のもたらす恵沢を確保し、" \
                    u"政府の行為によつて再び戦争の惨禍が起ることのないやうにすることを決意し、ここに主権が国民に存することを宣言し、" \
                    u"この憲法を確定する。そもそも国政は、国民の厳粛な信託によるものであつて、その権威は国民に由来し、" \
                    u"その権力は国民の代表者がこれを行使し、その福利は国民がこれを享受する。これは人類普遍の原理であり、" \
                    u"この憲法は、かかる原理に基くものである。われらは、これに反する一切の憲法、法令及び詔勅を排除する。"
    result = u"日本国民 は、 正当 に 選挙 された 国会 における 代表者 を 通じ て 行動 し、われらとわれらの 子孫 のために、 " \
             u"諸国民 との 協和 による 成果 と、わが 国 全土 にわたつて 自由 のもたらす 恵沢 を 確保 し、 政府 の 行為 によつて 再び 戦争 " \
             u"の 惨禍 が 起る ことのないやうにすることを 決意 し、ここに 主権 が 国民 に 存す ることを " \
             u"宣言 し、この 憲法 を 確定す る。そもそも 国政 は、 国民 の 厳粛 な 信託 によるものであつて、" \
             u"その 権威 は 国民 に 由来 し、その 権力 は 国民 の 代表者 がこれを 行使 し、その 福利 は 国民 " \
             u"がこれを 享受 する。これは 人類普遍 の 原理 であり、この 憲法 は、かかる 原理 に 基く ものである。われらは、" \
             u"これに 反す る 一切 の 憲法 、 法令 及び 詔勅 を 排除 する。"

    wakati = pykakasi.wakati()
    converter = wakati.getConverter()
    assert converter.do(original_text) == result
